/**************************************************************************//**
 * @file     main.c
 * @version  V3.00
 * @brief
 *           Show how to wake up system form SPD Power-down mode by Wake-up pin(PA.0)
 *           or Wake-up Timer or Wake-up ACMP or RTC Tick or RTC Alarm and RTC Tamper 0.
 *
 *
 * @copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "M480.h"
#include "IdleMode.h"
#include "hid_mouse.h"

#define SIGNATURE       0x125ab234
#define FLAG_ADDR       0x20000FFC
#define PLL_CLOCK_192       192000000
#define PLL_CLOCK_160       160000000

extern uint8_t volatile g_power_down;
extern uint8_t volatile g_reset;
extern int IsDebugFifoEmpty(void);

void PWRWU_IRQHandler(void)
{
    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));
	
	CLK->PWRCTL |= CLK_PWRCTL_PDWKIF_Msk;
	printf("\n PWRWU_IRQHandler");
}

void TMR0_IRQHandler(void)
{
    // Clear wake up flag
    TIMER_ClearWakeupFlag(TIMER0);
    // Clear interrupt flag
    TIMER_ClearIntFlag(TIMER0);
}

/*---------------------------------------------------------------------------------------------------------*/
/* ISR to handle USCI interrupt event                                                                      */
/*---------------------------------------------------------------------------------------------------------*/
void UART1_IRQHandler(void)
{
    uint32_t u32IntSts = UART1->INTSTS;
	
    if(u32IntSts & UART_INTSTS_WKIF_Msk) 
		{	
        printf("\n UART Wake-Up:  ");

        UART1->INTSTS = UART_INTSTS_WKIF_Msk;
				UART1->WKSTS = UART1->WKSTS;
    }
}

void WDT_IRQHandler(void)
{

    WDT_RESET_COUNTER();
	
    if(WDT_GET_TIMEOUT_INT_FLAG() == 1) {
        /* Clear WDT time-out interrupt flag */
        WDT_CLEAR_TIMEOUT_INT_FLAG();
		}
    if(WDT_GET_TIMEOUT_WAKEUP_FLAG() == 1) {
        /* Clear WDT time-out wake-up flag */
        WDT_CLEAR_TIMEOUT_WAKEUP_FLAG();
    }
		
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Turbo Mode, core voltage to 1.26V and                                    */
/*-----------------------------------------------------------------------------------------------------------*/
void  Turbo_Function()
{
		int coreVoltage = 1;

    /* Set PCLK0 = PCLK1 = HCLK/2 */
    CLK->CLKSEL0 |=0xc0;

    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));
	
		/* Select Core Voltage to 1.26V */
	  SYS_UnlockReg();
		//SYS->PLCTL |= (CLK->CLKSEL1 & 0xFFFFFFFC ) | (0x0);
		SYS->PLCTL &= 0xFFFFFFFC | (0x0);
		SYS->PLCTL |= 0x0;
		
		/* Check Core Voltage Change Busy */
		while(SYS->PLSTS & 0x1);
	
	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_192);

		SYS_LockReg();
	
		coreVoltage = 0x3 & SYS->PLSTS>>8;

		if(coreVoltage==0)
		{
			printf("LDO voltage:1.26V\n");
			printf("CPU freq:%d\n", CLK_GetCPUFreq());
		}
		else
			printf("Fail");

}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Normal run Mode and core voltage to 1.2V                                    */
/*-----------------------------------------------------------------------------------------------------------*/
void  Normal_Function()
{
		int coreVoltage = 1;

	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_160);

    /* Set PCLK0 = PCLK1 = HCLK/2 */
    CLK->CLKSEL0 |=0xc0;

    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));
	
		/* Select Core Voltage to 1.2V */
	  SYS_UnlockReg();
		SYS->PLCTL &= 0xFFFFFFFC | (0x0);
		SYS->PLCTL |= 0x1;
		
		/* Check Core Voltage Change Busy */
		while(SYS->PLSTS & 0x1);
	
	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_160);

		SYS_LockReg();
	
		/* Read Core Voltage */
		coreVoltage = 0x3 & SYS->PLSTS>>8;

		if(coreVoltage==1)
		{
			printf("LDO voltage:1.2V\n");
			printf("CPU freq:%d\n", CLK_GetCPUFreq());
		}
		else
			printf("Fail");

		/* Select Core Voltage to 1.26V */
	  SYS_UnlockReg();
		//SYS->PLCTL |= (CLK->CLKSEL1 & 0xFFFFFFFC ) | (0x0);
		SYS->PLCTL &= 0xFFFFFFFC | (0x0);
		SYS->PLCTL |= 0x0;
		
		/* Check Core Voltage Change Busy */
		while(SYS->PLSTS & 0x1);
		
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Fast Wake-up Power Down Mode and Wake up source by WDT                      */
/*-----------------------------------------------------------------------------------------------------------*/
void  FWUPDWakeUpWDTFunction(uint32_t u32PDMode)
{

    printf("Enter to Fast Wake-up Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));
	
		/* Enable WDT peripheral clock */
		CLK_EnableModuleClock(WDT_MODULE);
	
    /* Peripheral clock source */
    CLK_SetModuleClock(WDT_MODULE, CLK_CLKSEL1_WDTSEL_LIRC, 0);
	
    /* Enable WDT NVIC */
    NVIC_EnableIRQ(WDT_IRQn);
	
    /* Because of all bits can be written in WDT Control Register are write-protected;
       To program it needs to disable register protection first. */
    SYS_UnlockReg();
	
    /* Configure WDT settings and start WDT counting */
    WDT_Open(WDT_TIMEOUT_2POW14, WDT_RESET_DELAY_18CLK, TRUE, TRUE);

    /* Enable WDT interrupt function */
    WDT_EnableInt();

		/* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

}


/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by UART                                  */
/*-----------------------------------------------------------------------------------------------------------*/
void  PDWakeUpUARTFunction(uint32_t u32PDMode)
{
    printf("Enter to Normal Power-Down mode......\n");
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);
	
		/* Enable UART2 peripheral clock */
		CLK_EnableModuleClock(UART1_MODULE);
	
    /* Select UART module clock source as LIRC and UART module clock divider as 1 */
    CLK_SetModuleClock(UART1_MODULE, CLK_CLKSEL1_UART1SEL_LXT, CLK_CLKDIV0_UART1(1));

    /* Set PD multi-function pins for UART1 RXD and TXD  */
    SYS->GPA_MFPL &= ~(SYS_GPA_MFPL_PA2MFP_Msk | SYS_GPA_MFPL_PA3MFP_Msk);
    SYS->GPA_MFPL |= SYS_GPA_MFPL_PA2MFP_UART1_RXD;
    SYS->GPA_MFPL |= SYS_GPA_MFPL_PA3MFP_UART1_TXD;

    /* Reset UART module */
    SYS_ResetModule(UART1_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART1, 9600);

    /* Enable UART1 interrupt */
		UART1->WKCTL = UART_WKCTL_WKDATEN_Msk;
    NVIC_EnableIRQ(UART1_IRQn);
    UART_EnableInt(UART1, (UART_INTEN_WKIEN_Msk));

		CLK->PWRCTL |= CLK_PWRCTL_PDWKIEN_Msk;   
		NVIC_EnableIRQ(PWRWU_IRQn);

		UART1->WKSTS = UART1->WKSTS; // clecar status

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();
		
		printf("System has been waken up from power down mode .\n");

		UART_Close(UART1);
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by USB FS Device                         */
/*-----------------------------------------------------------------------------------------------------------*/
void  PDFunction_USBD(uint32_t u32PDMode)
{
    printf("Enter to Normal Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable External XTAL (4~24 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Switch HCLK clock source to HXT */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HXT,CLK_CLKDIV0_HCLK(1));

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(FREQ_192MHZ);

    /* Set both PCLK0 and PCLK1 as HCLK/2 */
    CLK->PCLKDIV = CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2;

    /* Select USBD */
    SYS->USBPHY = (SYS->USBPHY & ~SYS_USBPHY_USBROLE_Msk) | SYS_USBPHY_USBEN_Msk | SYS_USBPHY_SBO_Msk;

    /* Select IP clock source */
    CLK->CLKDIV0 = (CLK->CLKDIV0 & ~CLK_CLKDIV0_USBDIV_Msk) | CLK_CLKDIV0_USB(4);

    /* Enable IP clock */
    CLK_EnableModuleClock(USBD_MODULE);

    SYS->GPA_MFPH &= ~(SYS_GPA_MFPH_PA12MFP_Msk|SYS_GPA_MFPH_PA13MFP_Msk|SYS_GPA_MFPH_PA14MFP_Msk|SYS_GPA_MFPH_PA15MFP_Msk);
    SYS->GPA_MFPH |= (SYS_GPA_MFPH_PA12MFP_USB_VBUS|SYS_GPA_MFPH_PA13MFP_USB_D_N|SYS_GPA_MFPH_PA14MFP_USB_D_P|SYS_GPA_MFPH_PA15MFP_USB_OTG_ID);

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    USBD_Open(&gsInfo, HID_ClassRequest, NULL);

    /* Endpoint configuration */
    HID_Init();
    USBD_Start();
    /* Wakeup Enable */
    USBD->INTEN |= USBD_INTEN_WKEN_Msk;

    NVIC_EnableIRQ(USBD_IRQn);

        if (g_power_down && g_reset)
        {
            g_reset = 0;
            /* Check if all the debug messages are finished */
            UART_WAIT_TX_EMPTY(UART0);

            /* Enter to Power-down mode */
            CLK_PowerDown();
				
					  printf("System has been waken up from power down mode .\n");
					
        }
        //HID_UpdateMouseData();
				
		
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Low Leakage Power Down Mode and Wake up source by Wake-up Timer                         */
/*-----------------------------------------------------------------------------------------------------------*/
void  LLPDWakeUpTimerFunction(uint32_t u32PDMode, uint32_t u32Interval)
{

    printf("Enter to LLPD Power-Down mode......\n");

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));
	
		/* Enable Power wake-up interrupt */
		//CLK->PWRCTL |= CLK_PWRCTL_PDWKIEN_Msk;   
		//NVIC_EnableIRQ(PWRWU_IRQn);

    /* Select Timer clock source from LIRC */
    CLK_EnableModuleClock(TMR0_MODULE);

    CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0SEL_LIRC, 0);
		
    TIMER_Open(TIMER0, TIMER_PERIODIC_MODE, 1);
		
		/* Set Timer period: 10s */
		TIMER0->CMP = 100000;

    /* Enable timer wake up system */
    TIMER_EnableWakeup(TIMER0);

    /* Enable Timer0 interrupt */
    TIMER_EnableInt(TIMER0);
    NVIC_EnableIRQ(TMR0_IRQn);

    /* Start Timer0 counting */
    TIMER_Start(TIMER0);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

		printf("System has been waken up from Low Leakage Power Down Mode .\n");

		TIMER_Close(TIMER0);
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by GPIO Wake-up pin                    */
/*---------------------------------------------------------------------------------------------------------*/
void SPD0WakeUpPinFunction(uint32_t u32PDMode)
{
		// SRAM retention test
    M32(FLAG_ADDR) = SIGNATURE;
	
    printf("Enter to SPD0 Power-Down mode......\n");
		printf("SRAM retention:%d\n", M32(FLAG_ADDR));

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));
	
    /* Configure GPIO as Input mode */
    GPIO_SetMode(PC, BIT0, GPIO_MODE_INPUT);
	GPIO_SetMode(PB, BIT12, GPIO_MODE_INPUT);

    // GPIO SPD Power-down Wake-up Pin Select
    CLK_EnableSPDWKPin(2, 0, CLK_SPDWKPIN_RISING, CLK_SPDWKPIN_DEBOUNCEDIS);
	CLK_EnableSPDWKPin(1, 12, CLK_SPDWKPIN_RISING, CLK_SPDWKPIN_DEBOUNCEDIS);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Enter to Power-down mode */
    CLK_PowerDown();


    /* Wait for Power-down mode wake-up reset happen */
    while(1);
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Standby Power Down Mode and Wake up source by RTC Tick                              */
/*-----------------------------------------------------------------------------------------------------------*/
void  SPD1WakeUpRTCTickFunction(uint32_t u32PDMode)
{
	
    printf("Enter to SPD1 Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));
	
    /* enable RTC peripheral clock */
    CLK->APBCLK0 |= CLK_APBCLK0_RTCCKEN_Msk;
    /* RTC clock source select 32KHz */
    CLK->CLKSEL3 &= ~(CLK_CLKSEL3_RTCSEL_Msk);

    /* Open RTC and start counting */
    RTC->INIT = RTC_INIT_KEY;

    if(RTC->INIT != RTC_INIT_ACTIVE_Msk) {
        RTC->INIT = RTC_INIT_KEY;
        while(RTC->INIT != RTC_INIT_ACTIVE_Msk);
    }

    RTC_WaitAccessEnable();
    /* clear tick status */
    RTC_CLEAR_TICK_INT_FLAG();

    /* Enable RTC Tick interrupt */
    RTC_EnableInt(RTC_INTEN_TICKIEN_Msk);
    RTC_WaitAccessEnable();

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    RTC_SetTickPeriod(RTC_TICK_1_SEC);

    /* Enable RTC wake-up */
    CLK_ENABLE_RTCWK();
		
    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);		
		
    /* Enter to Power-down mode */
    CLK_PowerDown();

		RTC_Close();
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by Wake-up pin                         */
/*---------------------------------------------------------------------------------------------------------*/
void DPDWakeUpPC0Function(uint32_t u32PDMode, uint32_t u32EdgeType)
{
	
    printf("Enter to DPD Power-Down mode......\n");
		//printf("Wake up by PC.0 rising edg\n");

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));
	
    /* Configure GPIO as Input mode */
    GPIO_SetMode(PC, BIT0, GPIO_MODE_INPUT);

    // Set Wake-up pin trigger type at Deep Power down mode
    CLK_EnableDPDWKPin(u32EdgeType);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Enter to Power-down mode */
    CLK_PowerDown();

    /* Wait for Power-down mode wake-up reset happen */
    while(1);
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Turbo Mode, core voltage to 1.26V and                                    */
/*-----------------------------------------------------------------------------------------------------------*/
void  Turbo_Function_basic()
{
		int coreVoltage = 1;
	
    /* Set PCLK0 = PCLK1 = HCLK/2 */
    CLK->CLKSEL0 |=0xc0;

    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));
	
		/* Select Core Voltage to 1.26V */
	  SYS_UnlockReg();
		//SYS->PLCTL |= (CLK->CLKSEL1 & 0xFFFFFFFC ) | (0x0);
		SYS->PLCTL &= 0xFFFFFFFC | (0x0);
		SYS->PLCTL |= 0x0;
		
		/* Check Core Voltage Change Busy */
		while(SYS->PLSTS & 0x1);
	
	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_192);

		SYS_LockReg();
	
		coreVoltage = 0x3 & SYS->PLSTS>>8;

		if(coreVoltage==0)
		{
			printf("LDO voltage:1.26V\n");
			printf("CPU freq:%d\n", CLK_GetCPUFreq());
		}
		else
			printf("Fail");
		
	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);

}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Normal run Mode and core voltage to 1.2V                                    */
/*-----------------------------------------------------------------------------------------------------------*/
void  Normal_Function_basic()
{
		int coreVoltage = 1;

	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_160);

    /* Set PCLK0 = PCLK1 = HCLK/2 */
    CLK->CLKSEL0 |=0xc0;

    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));
	
		/* Select Core Voltage to 1.2V */
	  SYS_UnlockReg();
		SYS->PLCTL &= 0xFFFFFFFC | (0x0);
		SYS->PLCTL |= 0x1;
		
		/* Check Core Voltage Change Busy */
		while(SYS->PLSTS & 0x1);
	
	  /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK_160);

		SYS_LockReg();
	
		/* Read Core Voltage */
		coreVoltage = 0x3 & SYS->PLSTS>>8;

		if(coreVoltage==1)
		{
			printf("LDO voltage:1.2V\n");
			printf("CPU freq:%d\n", CLK_GetCPUFreq());
		}
		else
			printf("Fail");

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);	
		
}

void IDLE_Function_basic(void)
{
    printf("Enter to IDLE mode......\n");
	
    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));
	
	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
		SYS_UnlockReg();
		CLK_Idle();
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Fast Wake-up Power Down Mode and Wake up source by WDT                      */
/*-----------------------------------------------------------------------------------------------------------*/
void  FWUPDFunction_basic(uint32_t u32PDMode)
{

    printf("Enter to Fast Wake-up Power-Down mode......\n");
	
    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
		/* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

}


/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by UART                                  */
/*-----------------------------------------------------------------------------------------------------------*/
void  PDFunction_basic(uint32_t u32PDMode)
{
    printf("Enter to Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();
		
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Low Leakage Power Down Mode and Wake up source by Wake-up Timer                         */
/*-----------------------------------------------------------------------------------------------------------*/
void  LLPDFunction_basic(uint32_t u32PDMode, uint32_t u32Interval)
{

    printf("Enter to LLPD Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));

		UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

}

/*---------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by GPIO Wake-up pin                    */
/*---------------------------------------------------------------------------------------------------------*/
void SPD0Function_basic(uint32_t u32PDMode)
{
		// SRAM retention test
    M32(FLAG_ADDR) = SIGNATURE;
	
    printf("Enter to SPD0 Power-Down mode......\n");
		printf("SRAM retention:%d\n", M32(FLAG_ADDR));

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LXT, CLK_CLKDIV0_HCLK(1));

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Standby Power Down Mode and Wake up source by RTC Tick                              */
/*-----------------------------------------------------------------------------------------------------------*/
void  SPD1Function_basic(uint32_t u32PDMode)
{
	
    printf("Enter to SPD1 Power-Down mode......\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));

    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);		

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

}

/*---------------------------------------------------------------------------------------------------------*/
/*  Function for System Entry to Power Down Mode and Wake up source by Wake-up pin                         */
/*---------------------------------------------------------------------------------------------------------*/
void DPDFunction_basic(uint32_t u32PDMode, uint32_t u32EdgeType)
{
	
    printf("Enter to DPD Power-Down mode......\n");
		//printf("Wake up by PC.0 rising edg\n");

    /* To check if all the debug messages are finished */
    while(IsDebugFifoEmpty() == 0);
	
    /* Select Power-down mode */
    CLK_SetPowerDownMode(u32PDMode);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_LIRC, CLK_CLKDIV0_HCLK(1));

	  UART_Close(UART0);
		UART_Close(UART1);
		PDMA_Close(PDMA);
		EADC_Close(EADC);
	
    /* Enter to Power-down mode */
		SYS_UnlockReg();
    CLK_PowerDown();

    /* Wait for Power-down mode wake-up reset happen */
    while(1);
}

/*-----------------------------------------------------------------------------------------------------------*/
/*  Function for Check Power Manager Status                                                                  */
/*-----------------------------------------------------------------------------------------------------------*/
void CheckPowerSource(void)
{
    unsigned int uRegRstsrc;
    uRegRstsrc = CLK_GetPMUWKSrc();

    printf("Power manager Power Manager Status 0x%x\n", uRegRstsrc);

    if((uRegRstsrc & CLK_PMUSTS_ACMPWK_Msk) != 0)
        printf("Wake-up source is ACMP.\n");
    if((uRegRstsrc & CLK_PMUSTS_RTCWK_Msk) != 0)
        printf("Wake-up source is RTC.\n");
    if((uRegRstsrc & CLK_PMUSTS_TMRWK_Msk) != 0)
        printf("Wake-up source is SPD mode Wake-up Timer.\n");
    if((uRegRstsrc & CLK_PMUSTS_GPAWK_Msk) != 0)
        printf("Wake-up source is GPIO PortA.\n");

}

void SYS_Init(void)
{

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable HIRC, LXT clock (Internal RC 12MHz, external XTAL 32kHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk | CLK_PWRCTL_LIRCEN_Msk | CLK_PWRCTL_LXTEN_Msk);

    /* Wait for HIRC, LXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk | CLK_STATUS_LIRCSTB_Msk | CLK_STATUS_LXTSTB_Msk);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Enable HXT clock (external XTAL 12MHz) */
    //CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for HXT clock ready */
    //CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(192000000);

    /* Set PCLK0/PCLK1 to HCLK/2 */
    CLK->PCLKDIV = (CLK_PCLKDIV_PCLK0DIV2 | CLK_PCLKDIV_PCLK1DIV2);

    /* Enable IP clock */
    CLK_EnableModuleClock(UART0_MODULE);
		CLK_EnableModuleClock(UART1_MODULE);
    CLK_EnableModuleClock(PDMA_MODULE);
    CLK_EnableModuleClock(EADC_MODULE);
		CLK_EnableModuleClock(EPWM0_MODULE);

    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HIRC, CLK_CLKDIV0_UART0(1));
		CLK_SetModuleClock(UART1_MODULE, CLK_CLKSEL1_UART1SEL_HIRC, CLK_CLKDIV0_UART1(1));

    /* Select EPWM0 module clock source as PCLK0 */
    CLK_SetModuleClock(EPWM0_MODULE, CLK_CLKSEL2_EPWM0SEL_PCLK0, 0);

    /* EADC clock source is 80MHz, set divider to 8, ADC clock is 80/8 MHz */
    CLK_SetModuleClock(EADC_MODULE, 0, CLK_CLKDIV0_EADC(8));

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PD multi-function pins for UART0 RXD(PD.2) and TXD(PD.3) */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD2MFP_Msk | SYS_GPD_MFPL_PD3MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD2MFP_UART0_RXD | SYS_GPD_MFPL_PD3MFP_UART0_TXD);
		
    /* Set PA multi-function pins for UART1 TXD, RXD */
    SYS->GPA_MFPL &= ~(SYS_GPA_MFPL_PA2MFP_Msk | SYS_GPA_MFPL_PA3MFP_Msk);
    SYS->GPA_MFPL |= (SYS_GPA_MFPL_PA2MFP_UART1_RXD | SYS_GPA_MFPL_PA3MFP_UART1_TXD);
		
    /* Configure the GPB0 - GPB3 ADC analog input pins.  */
    SYS->GPB_MFPL &= ~(SYS_GPB_MFPL_PB5MFP_Msk);
    SYS->GPB_MFPL |= (SYS_GPB_MFPL_PB5MFP_EADC0_CH5);

    /* Disable the GPB0 - GPB3 digital input path to avoid the leakage current. */
    GPIO_DISABLE_DIGITAL_PATH(PB, BIT5);	
		
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable HIRC, LXT clock (Internal RC 12MHz, external XTAL 32kHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk | CLK_PWRCTL_LIRCEN_Msk | CLK_PWRCTL_LXTEN_Msk);

    /* Wait for HIRC, LXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk | CLK_STATUS_LIRCSTB_Msk | CLK_STATUS_LXTSTB_Msk);

    /* Select HCLK clock source as HIRC and HCLK source divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Enable HXT clock (external XTAL 12MHz) */
    //CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Wait for HXT clock ready */
    //CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source as HIRC and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HIRC, CLK_CLKDIV0_UART0(1));

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set PD multi-function pins for UART0 RXD and TXD  */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD2MFP_Msk | SYS_GPD_MFPL_PD3MFP_Msk);
    SYS->GPD_MFPL |= SYS_GPD_MFPL_PD3MFP_UART0_TXD;
    SYS->GPD_MFPL |= SYS_GPD_MFPL_PD2MFP_UART0_RXD;

    /* Set PC multi-function pins for CLKO(PC.13) */
    SYS->GPC_MFPH = (SYS->GPC_MFPH & ~SYS_GPC_MFPH_PC13MFP_Msk) | SYS_GPC_MFPH_PC13MFP_CLKO;

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART0 */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
	
    /* Set UART0 baud rate Compensation BRCOMP = 0xa5, when UART0 baud rate = 9600, UART0 clock source as LXT*/
    //UART0 -> BRCOMP = 0xa5;

}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{
    uint8_t u8Item;

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Release I/O hold status */
    CLK->IOPDCTL = 1;

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* ---------- Turn off RTC  -------- */
    CLK->APBCLK0 |= CLK_APBCLK0_RTCCKEN_Msk;
    RTC_WaitAccessEnable();
    RTC->INTEN = 0;
    RTC_Close();
    /* Init UART0 for printf */
    UART0_Init();

    /* Get power manager wake up source */
    //CheckPowerSource();

    /* To check if system has been reset by WDT time-out reset or not */
    if(WDT_GET_RESET_FLAG() == 1) {
        WDT_CLEAR_RESET_FLAG();
        printf("\n*** System has been reset by WDT time-out event ***\n");
    }
    else if(M32(FLAG_ADDR) == SIGNATURE) {
        printf("\nSystem waken-up from SPD0 mode done!\n");
				printf("SRAM retention:%d\n", M32(FLAG_ADDR));
        M32(FLAG_ADDR) = 0;
    } 
		//else
        //printf("\nSystem waken-up from SPD1 mode done!\n");


    CLK_EnableCKO(CLK_CLKSEL1_CLKOSEL_HCLK, 0, 1);	
		
		while(1){

    switch(u8Item) {
    case '1':	//Turbo mode.
        Turbo_Function();
        break;	
    case '2':	//Turbo mode.
        Normal_Function();
        break;	
    case '3':	//Idle mode Uart.
        IDLE_UARTFunction();
        break;			
    case '4':	//Fast Wake-up Power Down Wake-up Watch Dag.
        FWUPDWakeUpWDTFunction(CLK_PMUCTL_PDMSEL_FWPD);
        break;	
    case '5':	//Power Down Wake-up UART2 wake up.
        PDWakeUpUARTFunction(CLK_PMUCTL_PDMSEL_PD);
        break;
    case '6':	//Power Down Wake-up UART2 wake up.
        PDFunction_USBD(CLK_PMUCTL_PDMSEL_PD);
        break;		
    case '7':	//Low Leakage Power Down Wake-up TIMER time-out interval is 1024 OSC10K clocks.
        LLPDWakeUpTimerFunction(CLK_PMUCTL_PDMSEL_LLPD, CLK_PMUCTL_WKTMRIS_1024);
        break;
    case '8':	//SPD0 GPIO Wake-up pin(PA.0) and using rising edge wake up.
        SPD0WakeUpPinFunction(CLK_PMUCTL_PDMSEL_SPD0);
        break;
    case '9': //SPD1 Wake-up by RTC Tamper0, Low level.
        SPD1WakeUpRTCTickFunction(CLK_PMUCTL_PDMSEL_SPD1);
        break;
    case 'a':
        DPDWakeUpPC0Function(CLK_PMUCTL_PDMSEL_DPD, CLK_DPDWKPIN_RISING);
        break;
    case 'b':	//Turbo mode.
        Turbo_Function_basic();
        break;	
    case 'c':	//Turbo mode.
        Normal_Function_basic();
        break;	
    case 'd':	//Idle mode Uart.
        IDLE_Function_basic();
        break;			
    case 'e':	//Fast Wake-up Power Down Wake-up Watch Dag.
        FWUPDFunction_basic(CLK_PMUCTL_PDMSEL_FWPD);
        break;	
    case 'f':	//Power Down Wake-up UART2 wake up.
        PDFunction_basic(CLK_PMUCTL_PDMSEL_PD);
        break;		
    case 'g':	//Low Leakage Power Down Wake-up TIMER time-out interval is 1024 OSC10K clocks.
        LLPDFunction_basic(CLK_PMUCTL_PDMSEL_LLPD, CLK_PMUCTL_WKTMRIS_1024);
        break;
    case 'h':	//SPD0 GPIO Wake-up pin(PA.0) and using rising edge wake up.
        SPD0Function_basic(CLK_PMUCTL_PDMSEL_SPD0);
        break;
    case 'i': //SPD1 Wake-up by RTC Tamper0, Low level.
        SPD1Function_basic(CLK_PMUCTL_PDMSEL_SPD1);
        break;
    case 'j':
        DPDFunction_basic(CLK_PMUCTL_PDMSEL_DPD, CLK_DPDWKPIN_RISING);
        break;
    default:
        break;
     }

				
		printf("\n\nCPU @ %d Hz\n\n", SystemCoreClock);
    printf("+---------------------------------------------------------------------------+\n");
    printf("|    M480 Power-down Mode and Wake-up Sample Code                           |\n");
    printf("|    Please Select Power Down Mode and Wake up source.                      |\n");
    printf("+---------------------------------------------------------------------------+\n");
    printf("|[1] Turbo mode with Core Freq 192MHz and LDO 1.26V.                        |\n");
    printf("|[2] Normal mode with Core Freq 160MHz and LDO 1.2V.                        |\n");
    printf("|[3] IDLE mode with Uart0 transfer ADC5(PIN1) data(8 bits) by PDMA.         |\n");
    printf("|[4] Fast wake up power down mode with LIRC and WDT wakeup after 10s.       |\n");
    printf("|[5] Power down mode with LXT and UART(PIN66) wakeup.                       |\n");
		printf("|[6] Power down mode with USB D+,D- wakeup.                                 |\n"); 
    printf("|[7] Low Leakage power down mode with LIRC and Timer wakeup after 10s.      |\n");
    printf("|[8] SPD0 Wake-up by GPIO PC.0(PIN82) with data reteantion.                 |\n");
    printf("|[9] SPD1 Wake-up by RTC Alarm after 1s.                                    |\n");
    printf("|[a] DPD1 Wake-up by GPIO PC.0(PIN82).                                      |\n");
    printf("|[b] Turbo mode with Core Freq 192MHz, LDO 1.26V and other function closed. |\n");
    printf("|[c] Normal mode with Core Freq 160MHz, LDO 1.2V and other function closed. |\n");
    printf("|[d] IDLE mode with other function closed.                                  |\n");
    printf("|[e] Fast wake up power down mode with other function closed.               |\n");
    printf("|[f] Power down mode with other function closed.                            |\n");
    printf("|[g] Low Leakage power down mode with other function closed.                |\n");
    printf("|[h] SPD0 with other function closed.                                       |\n");
    printf("|[i] SPD1 with other function closed.                                       |\n");
    printf("|[j] DPD1 with other function closed.                                       |\n");
    printf("+---------------------------------------------------------------------------+\n");

    /* To check if all the debug messages are finished */

		u8Item = getchar();
		
    while(IsDebugFifoEmpty() == 0);
		 
	}
}

